#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module classes
#
# Copyright (C) 2017 Univention GmbH <http://www.univention.de/>
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import sys
import traceback
import logging
import listener
import univention.debug as ud


class OXLogger(object):
	_ox_debug_level = int(listener.configRegistry.get('ox/debug/level', ud.ERROR))
	_listener_debug_level = int(listener.configRegistry.get('listener/debug/level', ud.PROCESS))
	_stdout_logger = None
	_UDEBUG_TO_LOGGING = {
		ud.ERROR: 'error',
		ud.WARN: 'warn',
		ud.PROCESS: 'info',
		ud.INFO: 'info',
		ud.ALL: 'debug',
	}

	def __init__(self, name=None, qualifier=None):  # type: (str, Optional[str]) -> None
		self._module_name = name or 'OX'
		self._qualifier = qualifier
		ud.init("/var/log/univention/listener.log", ud.NO_FLUSH, ud.FUNCTION)
		ud.set_level(ud.LISTENER, int(self._listener_debug_level))
		self._stdout = False

	@property
	def stdout(self):  # type: () -> bool
		return self._stdout

	@stdout.setter
	def stdout(self, log_to_stdout):  # type: (bool) -> None
		self._stdout = log_to_stdout
		if self._stdout and not self._stdout_logger:
			self._stdout_logger = logging.getLogger()
			self._stdout_logger.setLevel(logging.DEBUG)
			if 'ucs-test' not in [h.get_name() for h in self._stdout_logger.handlers]:
				handler = logging.StreamHandler()
				handler.set_name('ucs-test')
				handler.setFormatter(logging.Formatter(
					fmt='%(asctime)s %(levelname)s: %(message)s',
					datefmt='%Y-%m-%d %H:%M:%S'))
				handler.setLevel(logging.DEBUG)
				self._stdout_logger.addHandler(handler)
		else:
			self._stdout_logger = None

	@property
	def module_name(self):  # type: () -> str
		return self._module_name

	@module_name.setter
	def module_name(self, name):  # type: (str) -> None
		self._module_name = name

	@property
	def qualifier(self):  # type: () -> str
		return self._qualifier

	@qualifier.setter
	def qualifier(self, qualifier):  # type: (str) -> None
		self._qualifier = qualifier

	def emit(self, level, msg):  # type: (str, str) -> None
		if self._ox_debug_level > self._listener_debug_level:
			level = min(level, self._listener_debug_level)
		ud.debug(ud.LISTENER, level, '{}{}: {}'.format(
			self._module_name,
			'({})'.format(self.qualifier) if self.qualifier else '',
			msg))
		if self.stdout:
			getattr(self._stdout_logger, self._UDEBUG_TO_LOGGING[level])(msg)

	def debug(self, msg):  # type: (str) -> None
		self.emit(ud.ALL, msg)

	def error(self, msg):  # type: (str) -> None
		self.emit(ud.ERROR, msg)

	def exception(self, msg):  # type: (str) -> None
		exc_type, exc_value, exc_traceback = sys.exc_info()
		tb_s = traceback.format_exception(exc_type, exc_value, exc_traceback)
		self.error('{}\n{}'.format(msg, tb_s))

	def info(self, msg):  # type: (str) -> None
		self.emit(ud.INFO, msg)

	def process(self, msg):  # type: (str) -> None
		self.emit(ud.PROCESS, msg)

	def warn(self, msg):  # type: (str) -> None
		self.emit(ud.WARN, msg)
